#include "hardware/gpio.h"
#include "hardware/i2c.h"

#include "EA_TFT.h"
#include "EA_TOUCH_def.h"

//-------------------------------------------------------------------------------------------------
// global variables

int16_t gX, gY;
int16_t gTouchEvent = TOUCH_UP;
int16_t gTouchOK    = 0;

//-------------------------------------------------------------------------------------------------

int16_t read_FT3267(int16_t *pX, int16_t *pY)
{
  uint8_t buf[8];

  buf[0] = 2;
  i2c_write_blocking(TOUCH_PORT, 0x38, buf, 1, true);   // Set FT3267 Register 2
  i2c_read_blocking( TOUCH_PORT, 0x38, buf, 5, false);  // read Register 2..6

  if(buf[0] > 0)
   { *pX = ((buf[1]&0x0F) << 4) + buf[2];
     *pY = ((buf[3]&0x0F) << 4) + buf[4];
     return 1;
   }

  return 0;
}

//-------------------------------------------------------------------------------------------------

int16_t read_ST1615(int16_t *pX, int16_t *pY)
{
  uint8_t buf[4];

  buf[0] = 0x12;
  i2c_write_blocking(TOUCH_PORT, 0x55, buf, 1, true);   // Set ST16157 Register 0x12
  i2c_read_blocking( TOUCH_PORT, 0x55, buf, 3, false);  // read Point 1

  if(buf[0] & 0x80)   // coordinates valid ?
   { *pX = ((buf[0]&0x70) << 4) + buf[1];
     *pY = ((buf[0]&0x07) << 8) + buf[2];
     return 1;
   }

  return 0;
}

//-------------------------------------------------------------------------------------------------

int16_t read_GT911(int16_t *pX, int16_t *pY)
{
  uint8_t buf[8];

  buf[0]=0x81; buf[1]=0x4E;
  i2c_write_blocking(TOUCH_PORT, 0x5D, buf, 2, true);   // Set GT911 Registeradr 0x814E
  i2c_read_blocking( TOUCH_PORT, 0x5D, buf, 1, false);  // read buffer status

  if(buf[0] & 0x80)       // coordinates available ?
   { int16_t pointanz = buf[0] & 0x0F;
     if(pointanz > 0)
      { i2c_read_blocking(TOUCH_PORT, 0x5D, buf, 5, true);  // read id and Point 1
        *pX = buf[1] + (buf[2] << 8);
        *pY = buf[3] + (buf[4] << 8);
      }
     buf[0]=0x81; buf[1]=0x4E;  buf[2]=0;
     i2c_write_blocking(TOUCH_PORT, 0x5D, buf, 3, false);   // clear buffer status -> set Register 0x814E = 0
     return pointanz;
   }

   return -1;
}

//-------------------------------------------------------------------------------------------------

void correctCoordinates(uint16_t *pX, uint16_t *pY)
{
  uint16_t x = *pX;
  uint16_t y = *pY;

  switch(gTFTAngle)
   { case 90:   *pX = gTFTwidth-1-y;  *pY = x;               break;
     case 180:  *pX = gTFTwidth-1-x;  *pY = gTFTheight-1-y;  break;
     case 270:  *pX = y;              *pY = gTFTheight-1-x;  break;
   }
}

//-------------------------------------------------------------------------------------------------

void gpio_callback(uint gpio, uint32_t events)
{
  if(gpio == TOUCH_INT)
   { int16_t x, y, pointanz = -1;
     switch(gTFTmodul)
      { case EA_TFT009: pointanz = read_FT3267(&x, &y);  break;
        case EA_TFT015: pointanz = read_ST1615(&x, &y);  break;
        default:        pointanz = read_GT911(&x, &y);
      }
     if(pointanz >= 0)
      { if(pointanz == 0)
         { if(gTouchEvent != TOUCH_UP)
            { gTouchEvent = TOUCH_UP;
              gTouchOK    = 1;
            }
         }
        else
         { gTouchEvent = (gTouchEvent == TOUCH_UP) ? TOUCH_DOWN : TOUCH_DRAG;
           if(gTFTAngle != 0)
              correctCoordinates(&x, &y);
           if(gTouchEvent == TOUCH_DOWN  ||  x != gX  ||  y != gY)
            { gX = x;
              gY = y;
              gTouchOK = 1;
            }
         }
      }
   }
}

//-------------------------------------------------------------------------------------------------

void TOUCHi2c_init(uint clk)
{
    i2c_init(TOUCH_PORT, clk);

    gpio_set_function(TOUCH_SDA, GPIO_FUNC_I2C);
    gpio_set_function(TOUCH_SCL, GPIO_FUNC_I2C);
    gpio_pull_up(TOUCH_SDA);
    gpio_pull_up(TOUCH_SCL);
    
    gpio_set_function(TOUCH_INT, GPIO_FUNC_SIO);
    gpio_set_dir(TOUCH_INT, GPIO_IN);
    gpio_pull_up(TOUCH_INT);

    gpio_set_irq_enabled_with_callback(TOUCH_INT, GPIO_IRQ_EDGE_FALL, true, &gpio_callback);
}

//-------------------------------------------------------------------------------------------------

void waitUntilTouch(int16_t mask)
{
  while(1)
   { if(gTouchOK)
      { gTouchOK = 0;
        if(mask & gTouchEvent)
           break;
      }
   }
}
